function Y = tsexp(T,alpha,t0)

% TSEXP     Timescale Exponential.
%   TSEXP(T,ALPHA,t0) returns the exponential of the input timescale.
%   Required inputs are T (timescale object), ALPHA (exponent) and t0
%   (initial condition). t0 must be contained in T. If there is no input for t0,
%   the first value of the timescale is used as the intial value. If t0 is not the first term of
%   the timescale, the exponential (Y) of all values before t0 is set to zero.
%   Uniform discrete values are calculated as follows:
%   Y(i) =(1 + MU(i)*ALPHA)^((T-t0)/MU(i))
%   This is when for a timescale hZ, h is constant.
%   Non-uniform discrete exponents are calculated using
%   recursion. 
%   Continuous exponents are computed using MATLAB's exponential
%   function as follows:
%   Y(i) = POWER(EXP(T(:)),ALPHA)

%Brian Ballard, Bunmi Otegbade 06/17/05

% ERROR CHECKING

% Prepare output: Create an output timescale filled with zeros.
type = size(T);
Y = timescale(zeros(1,length(T)),type);
empty = 0;

% Checking number of arguments
    if(nargin < 2)
        error('Requires at least 2 arguments.')
    elseif(nargin > 3)
        error('Too many arguments')
    end
    
% CASE: Only two input arguments
    if(nargin == 2)
    
        % Checking if T is a valid timescale 
        if(~isa(T,'timescale'))
            error('First argument must be a timescale object');
        end
        
        % Checking if alpha is numeric
        if(~isnumeric(alpha))
            error('The exponent must be a number');
        end
        
        % Get the value of t0 from the timescale
        if (type(1)==0)
            error('Cannot compute the exponential of an empty interval and only last interval can be empty.')
        else
            t0 = T(1);
        end
        
% CASE: Three input arguments
    elseif(nargin == 3)
       
        if(~isa(T,'timescale'))
            error('First argument must be a timescale object');
        end
    
        if(~isnumeric(alpha))
            error('The exponent must be a number');
        end

        if(~isnumeric(t0))
            error('The initial condition must be a number');
        end
        
        % Error Check for t0 in the timescale
        p = 1;
        in_timescale = 0;
        for i = 1:size(type,2)      % from 1 to the number of intervals
            q = type(1,i);          % index of last element of that interval
                
            %if DISCRETE interval
            if (type(2,i)) 
                x = T(:,i) - t0;
                absx = abs(x);
                avgt0 = sum(absx) / length(absx);
                   if(~(min(absx) >= (100000000000000 * eps * avgt0)))
                        in_timescale = 1;   % the initial value is in the timescale
                        break;
                   end
            %end DISCRETE interval

            %if CONTINUOUS interval   
            else
                if(isempty(T(:,i)))%if empty interval
                    in_timescale = 0;
                else
                    if( T(p) <= t0 & T(q) >= t0 )
                        in_timescale = 1; 
                        break;
                    end
                end
            end
            p = q + 1;
        end %End the loop
        
        if(~in_timescale)
            error('The initial condition must be inside the timescale object')
        end
    end %End number of arguments

%This loop cycles through each interval, determining if the interval is
%discrete, continuous or empty. If discrete, uniform and the first
%interval, it calculates the exponential in hZ using the formula described
%by Bohner-Peterson. If discrete and non uniform, it calculates the
%exponential using recursion; when it is the first interval, it starts the
%recursion at 1. If continuous, it calculates the usual exponential in R. 
%Between intervals, a "glue" is calculated by recursion to determine the 
%starting point of the next interval. Output is done in real time.

in_timescale =0;
p = 1;
for i = 1:size(type,2)      % from 1 to the number of intervals
    q = type(1,i);          % index of last element of that interval (think RC)
    if (i == 1) hold = 1; end
    
    %Check if EMPTY interval
    if( isempty(T(:,i)) )
        empty = 1;
        % Only last interval can be empty
        if((i~=size(type,2))) %if last interval,
            error('Only the last interval can be empty');
        end
    end
    %end EMPTY interval
        
    %if DISCRETE interval
    if (type(2,i)) 
                
        v = T(:,i); % Strip members of interval into a vector 
        mu = diff(v); % Get the graininess vector
       
        % Check if mu is constant?
        if(empty==1)
            empty=1;
        else
            avg = sum(mu)/length(mu);
            mu2 = abs(mu - avg);
            if(max(mu2) > 10000000000 * eps * avg)
                const = 0;
            else
                const = 1;
            end
        end
        
        if(in_timescale == 0) %WHILE THE FLAG IS STILL ZERO t0 has not been found
            % IS T0 in it? Set flags and put zeros in Y until t0 is found.
            for j=p:q      %Go through each member of the interval.
                if( ( T(j-p+1,i) - t0 ) < 0 )   %if the difference is negative,
                   Y(j-p+1,i) = 0;
                   in_timescale = 0; 
                   hold = 1;last = T(q); j=j-p+1;
                end                    
                if( abs( (T(j-p+1,i)-t0) ) <= 1000 * eps * avg)
                    Y(j-p+1,i) = 1;in_timescale = 1;z=i;
                    j=j-p+1;
                    break;
                end
            end
        end
     
        % Is this the first interval(DISCRETE)
        if(i < 2)
                
                if(in_timescale == 0)
                    %DO nothing                    
                   
                else %(in_timescale == 1)if in the interval,
                    %******************** CALCULATIONS ********************************
                    % Is mu constant?
                    if (const == 1) % NOTE here, we overwrite zeros in the result
                        Y(j+1:(q-p+1),i) = (1+mu(1)*alpha).^((v(j+1:q-p+1)-t0)/mu(1)); % t0 or T(j,i)
                        hold = Y((q-p+1),i);        %Hold last exponent value of interval
                        last = T(q);          %Hold the last t value of interval
                    else%(not constant)
                         
                        if(empty==1)        %because first = undef
                            break;
                        else 
                            first = T(p); 
                        end
                        
                        Y(j+1:(q-p+1),i) = recursive(p+j,q,mu(j:q-p),hold,alpha);    % assuming a recursive function
                        hold = Y((q-p+1),i);    %Again, hold the last exponent value
                        last = T(q);            %Hold the last t value
                    end
                end %end t0 in timescale or not
          
        else %(ie: not the first interval DISCRETE)
               
                %Check if t0 has been found
                if(in_timescale == 0)
                    %DO nothing
                                        
                else%(in_timescale == 1)
                        %******************** CALCULATIONS ********************************
                        if (const == 1)
                            if(empty==1) % Check if empty, because first is undef
                                break;
                            else 
                                first = T(p); 
                            end
                            
                            if(z == i) %if t0 comes in it, calculate this way
                                hold = 1;              
                                Y(j+1:(q-p+1),i) = hold*(1+mu(1)*alpha).^((v(j+1:(q-p+1))-t0)/mu(1)) ;%t0 or T(j,i)
                                v(j+1:(q-p+1)),T(j,i)
                                hold = Y((q-p+1),i);        %Hold last exponent value of interval
                                last = T(q);          %Hold the last t value of interval
                            end
                            
                            if(z < i) %if full interval without t0 in it, calculate this way
                                mul = first - last;
                                hold = hold * (1 + mul*alpha);              
                                Y(1,i) = hold; 
                                Y(2:(q-p+1),i) = hold*(1+mu(1)*alpha).^((v(2:(q-p+1))-T(p))/mu(1)) ; % full int so T(p)
                                hold = Y((q-p+1),i);        %Hold last exponent value of interval
                                last = T(q);          %Hold the last t value of interval
                            end
                            
                        else %(const ~= 0)
                            if(empty==1) % Check if empty, because first is undef
                                break;
                            else 
                                first = T(p); 
                            end
                            
                            % z determines if t0 was found earlier(1) or has just
                            % been found(0).
                             
                            if(z == i) %if t0 comes in it, calculate this way
                                hold = 1;
                                Y(j+1:(q-p+1),i) = recursive(p+j,q,mu(j:q-p),hold,alpha);    % assuming a recursive function
                                hold = Y((q-p+1),i);    %Again, hold the last exponent value
                                last = T(q);            %Hold the last t value
                            end

                            if(z < i) %if full interval without t0 in it, calculate this way
                                mul = first - last; % last t of previous interval - first t of current interval
                                hold = hold * (1 + mul*alpha);                      % Our old mistake here was using
                                Y(1,i) = hold;                                      % both p-q indexing and interval indexing
                                Y(2:(q-p+1),i) = recursive(p+1,q,mu,hold,alpha);    % assuming a recursive function
                                hold = Y((q-p+1),i);    %Again, hold the last exponent value
                                last = T(q);            %Hold the last t value
                            end
                        end %end const or not (25)
                end %end timescale in or not (17)
        end %end first or not(9)
           
    %if CONTINUOUS interval   
    else
        
        
        if(in_timescale == 0) %WHILE THE FLAG IS STILL ZERO -t0 has not been found
            % IS T0 in it? SET FLAGS ONLY
            for j=p:q      %Go through each member
                if( ( T(j-p+1,i) - t0 ) < 0 )   %if the difference is negative,
                   Y(j-p+1,i) = 0;
                   in_timescale = 0; 
                   hold = 1;last = T(q); j=j-p+1;
                end                    
                if( abs( (T(j-p+1,i)-t0) ) <= 1000 * eps * 100 )
                    Y(j-p+1,i) = 1;in_timescale = 1;z=i;
                    j=j-p+1;
                    break;
                end
            end
        end
        
   
        if (i==1) %if first interval
                if(in_timescale == 0)
                    %Do nothing
                else%(in_timescale==1)
                    Y(j+1:(q-p+1),i)=power(exp(T(j+1:q-p+1) - t0),alpha); %Calculate exponential in R        
                end
        else %ie not first interval
                if(in_timescale == 0 )
                    %Do nothing
                else %(in_timescale ==1)
                    % Check if next interval is empty because first = undef           
                    
                    
                    if(z == i) %if t0 comes in it
                        if(empty==1) 
                            break;
                        else 
                            first = T(p); 
                        end
                        hold = 1;                        
                        Y(j+1:(q-p+1),i) = hold * power(exp(T(j+1:q-p+1) - t0),alpha); %t0 or T(j,i)
                    end
                    
                    
                    if(z < i) %if full interval without t0 in it
                        if(empty==1) 
                            break;
                        else 
                            first = T(p); 
                        end

                        mul = first - last;     % last t of previous interval - first t of current interval
                        hold = hold * (1 + mul*alpha); % Calculate glue
                        Y(1,i) = hold;        % Keep first value
                        Y(2:(q-p+1),i) = hold * power(exp(T((p+1):q) - T(p)),alpha); %this MUST be T(p)
                    end
                end
        end      
        hold = Y((q-p+1),i);                %Y{last t of continuous interval};
        last = T(q);                    %last t of continuous interval;
        
    end %DISCRETE OR CONTINUOUS (5)    
    p = q + 1;                          %p and q are only indices    
end %biggest loop